/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*******************************************************************************
 *              Copyright 2000,2001,2002,2003 Element 14 Inc                   *
 *                                                                             *
 * This material is the confidential trade secret and proprietary information  *
 * of Element 14 Inc.  It may not be reproduced, used, sold or transferred to  *
 * any third party  without the prior written  consent of Element 14 Inc. All  *
 * rights reserved.                                                            *
 *                                                                             *
 ******************************************************************************/
/* 
   $Id: master.h,v 1.4 2004/10/25 14:45:26 bchristi Exp $
*/
#ifndef MASTER_H
#define MASTER_H

#include "bcmtypes.h"
#include "bcmatmapi.h"

#include <linux/time.h>
typedef struct timeval timeval;
#include <asm/mipsregs.h>


#define SID_MASK 0xFFF /* we use default 12 bits sids */

#define ATM_HEADER_BYTES  4
/* ATM_CELL_SIZE should only be defined this way when compiling the
master */
#ifdef ATM_BONDING_MASTER
#define ATM_CELL_SIZE (ATM_HEADER_BYTES + 48)
#endif

#define MAX_TIME_INTERVAL 1000 /* max time step for the virtual time */
#define TIME_BETWEEN_CPE_POLL 150000 /* time between two getState requests */
//#define ADD_TX_DELAY  13000 /* 10msec is ok, 5 msec  seems to be ok too, add 10 msec to cover LAN jitter */
#define ADD_TX_DELAY  0 /* 10msec is ok, 5 msec  seems to be ok too, add 10 msec to cover LAN jitter */

#define BDID_RESET_TIMEOUT 100000 /* time after which the ASM with BdId 0 will be
                              * considered as lost */

#define CPE_DOWN_TIMEOUT  200000000 /* in usec */
#if 1
#define ASM_DOWN_TIMEOUT  (2*TIME_BETWEEN_CPE_POLL*MAX_BONDED_LINES) /* in usec */
#else
#define ASM_DOWN_TIMEOUT  (2000000) /* in usec */
#endif
/* ASM header in line format (i.e. in bigendian)
   Usual VPVC in local format

*/
#if 1 /* put 0 to avoid use of bonding at co side */
#define ASM_HEADER    0x142 /* include PTI bit AAL5 termination */
#define DEFAULT_VPVC  0x023 /*0x1000023*//* 0x023  vp=0, vc=35 */
#else
#define ASM_HEADER       0x232 /* include PTI termination bit */
#define DEFAULT_VPVC   0x10023 /* vp=1, vc=35 */
#endif

//#define MAX_BONDED_LINES 6
#define MAX_BONDED_LINES 2
/* message definitions */

typedef struct MacAddress MacAddress;
struct MacAddress 
{
  uint16 mac[3];
};


#define ETH_GET_STATE_REQ (0x831)
#define ETH_GET_STATE_REP (0x832)
#define ETH_GET_DEVICE_INFO_REQ ETH_GET_STATE_REQ
#define ETH_GET_DEVICE_INFO_REP ETH_GET_STATE_REP
#define ETH_ATM_CELL_FW   (0x833)
#define ETH_PRINT         (0x834)
#define ETH_REMOTE_CONSOLE (0x835)
#define ETH_MULTI_ATM_CELL_FW (0x836)
#define ETH_FORWARD_AAL5      (0x837)
#define ETH_MULTI_FWD_AAL5 (0x838) /* only used for DS */
#define ETH_IMAGE_UPGRADE_REP (0x0839)
#define ETH_IMAGE_UPGRADE_REQ (0x083A)
#define ETH_STATE_CHANGE_NOTIFICATION (0x083C)

typedef struct EthHeader EthHeader;
struct EthHeader
{
  MacAddress destination;
  MacAddress source;
  uint16     etherType;
  uint16     dummy ;
};



typedef struct AtmPayload AtmPayload;
struct AtmPayload 
{
  uint8  payload[48];
};

typedef struct AtmCell AtmCell;
struct AtmCell
{
  uint8      celldata[56]; /* round up the ATM cell size. With 4 bytes header,
                            * it could be sufficient to use 52 bytes (48+4) */
};


#define ATM_TX_QUEUE_LEN_LOG2 7
#define ATM_TX_QUEUE_LEN  (1<<ATM_TX_QUEUE_LEN_LOG2)
#define ATM_TX_QUEUE_INDEX_MASK (ATM_TX_QUEUE_LEN-1)
#define MAX_TX_CELLS  120

typedef struct CellInfo CellInfo;
struct CellInfo 
{
  int32    sidOffset;
  uint32   asmOffset;
};

typedef struct AtmCellEncaps AtmCellEncaps; /* 64 bytes */
struct AtmCellEncaps
{
  uint32 cellOffset; /* offset of the cell compared to AtmCellEncaps */
  uint32 sidOffset; /* offset of the sid compared to the sid of the first cell
                     * in the buffer -- ~0 means ASM cell */
  AtmCell cell; /* 56 bytes */
  uint32 dummy0;
  uint32 dummy1;
};


typedef struct MultiAtmCellHeader MultiAtmCellHeader;
struct MultiAtmCellHeader
{
  EthHeader ethHeader;
  uint8 cellQuantity; /* number of effective non asm cells in the buffer */
  uint8 totalCells; /* determine the exact message size = (lastCellIdx+1)*68+20 */
  uint8 asmIndex; /* ~0 means no asm in buffer */
  uint8 dummy; /* offset in byte compared to frame pointer */
  uint16 firstSid; /* provide 64 bit align */
  uint16 maxSidOffet; /* provide 64 bit align */
};


#define CELL_AGGR 20
typedef struct MultiAtmCell MultiAtmCell;
struct MultiAtmCell
{
  MultiAtmCellHeader hdr; /* 20 bytes */
  AtmCellEncaps      atm[CELL_AGGR];
};

#define MAX_AAL5_CELLS 30
#define AAL5_FWD_WRONG_CRC_FLAG (1 << 0)
#define AAL5_FWD_TRAILER_FLAG   (1 << 1)
typedef struct AAL5forward AAL5forward;
struct AAL5forward
{
  EthHeader ethHeader;
  uint16    totalMsgSize; /* expressed in bytes */
  uint16    vpvcIndex;       /* VP/VC (main ly important for US traffic ? */  
  uint8     startFrame; /* 1 when contains the first part of an AAL5 frame */
  uint8     endFrame;   /* 1 when contains the end of an AAL5 frame. Note in
                           case endFrame is 0, the message contains 30*48
                           bytes. Other wise, it contains bytes till the end
                           of the message. */
  uint8     flags;
  uint8     summy;
  AtmPayload   aal5Frame[MAX_AAL5_CELLS];
};

#define AAL5FORWARD_SIZE (sizeof(AAL5forward))
#define AAL5FORWARD_HEADER_SIZE (AAL5FORWARD_SIZE - MAX_AAL5_CELLS * sizeof(AtmPayload))

/* data in MultiAal5Fwd is a sequence of aal5 frames.
   Each frame is preceded by a size information (2 bytes). and 2 alignement bytes
   frame is placed in a message, end of the frame will be placed in the next message

   !! note - offset is rounded up after each frame.
           - some dummy byte can be added at the end of a frame
   
   Msg0 = N %% dummy  %% frame0 (N bytes) %% M %% dummy %% frame1 (M bytes) %% K %% dummy %% frame2 (L firstbytes)
   Msg1 = K-L %% dummy %% frame2 (K-L last bytes) %% T %% dummy %% frame3 (T bytes) 
   Msg2 = S %% dummy %% frame4 (S bytes)


   How to process the message:
    - offset = 0
    - while (offset < msgSize)
    -   frameSize = *(uint16 *)(data+offset)
    -   ofsset += 4
    -   if pendingBuffer ==  NULL
          pendingBuffer = allocate buffer of frameSize
          pendingOffset = 0
        else
          if expectedSize != PendingSize
             pendingOffset = 0 (try to reuse the previous buffer if possible
             !!! new size is different from previously used size
             
    -   availSize = MIN(msgSize-offset,frameSize)
    -   copy(pendingBuffer+pendingOffset, data+offset, availSize)
    -   pendingOffset = availSize
    -   if (availSize == frameSize)
             send pendingBuffer; pendingBuffer = NULL
        else
             expectedSize = frameSize - availSize
             
    -   offset += (availSize+3)&~3 (keep offset multiple of four)
*/

#define MULTI_AAL5_SIZE (1500-16-4)
typedef struct MultiAal5Fwd MultiAal5Fwd;
struct MultiAal5Fwd
{
  EthHeader ethHeader;
  uint16    seqId;
  uint32    vpvc;
  uint8    data[MULTI_AAL5_SIZE]; 
};

#define MULTI_AAL5_HEADER_SIZE (sizeof(MultiAal5Fwd) - sizeof(uint8 [MULTI_AAL5_SIZE]))


typedef struct TxAAL5 TxAAL5;
struct TxAAL5 
{
  AtmPayload  cells[ATM_TX_QUEUE_LEN];
  uint8       endBitMap[ATM_TX_QUEUE_LEN]; /* array index to ATM header */
  uint32      vpvc[2]; /* currently only 1 vp/vc supported, so we only have
                        * two types of header (normal packet and end packet) */
  int         nextToSend; /* index to next to send */
  int         nextFree;   /* index to next free cell */
  uint32      headerCrc;
  uint32      maxCells;
  uint32      sid;
  uint32      sidMask; /* allow to mask on 8 bits or 12 bits */
  int         ethFrameCount;
  int         discardedEthFrames;
  int         headerSize;
  
  /* how to map sid into the atm header (network order)*/
  /*   |--------|3++0----|----7++4|11+8----| **
  **   |31    24|23    16|15     8|7      0| */
  
};

void TxAAL5_init(TxAAL5 *objp,
                 uint32 maxCells,
                 uint32 vpvc[] /* vp vc 2 byte vp, 2 bytes vc */,
                 uint32 vpvc_number, uint32 sidMask, int addAal5Header);
 /* change queue length, potentially drop oldest frame  => not used in a first
  * phase. Note it would be better to drop the newest frame but ... it easier
    to drop the oldest frame */
void TxAAL5_changeMaxCells(TxAAL5 *objp, uint32 maxCells);
/* compute CRC + AAL5 encapsulation, drop the frame if no place */
void TxAAL5_sendFrame(TxAAL5 *objp, uint8 *framep /* must allow to add 8 bytes
                                                   * upfront/ should be long
                                                     aligned + 32 bit aligned */,
                      int length, uint32 vpvcIndex);


uint32 Tx_AAL5_getNextATMcell(TxAAL5 *objp,uint8 **payloadp);



#define MAX_AAL5_LEN 33 /* length in ATM cells */

typedef struct RxAAL5 RxAAL5;
struct RxAAL5 
{
  uint32      vpvc; /* vpvc currently allocated with this RxAAL5 */
  AtmPayload  *aal5Frame;
  int         nextFree; /* where to place next ATMcell payload */  
  int wrongVPVC;
  int wrongLen;
  int wrongCRC;
  int correctFrames;
  int totalCells;
};

void RxAAL5_init(RxAAL5 *queue,uint32 vpvc);
void RxAAL5_pushCell(RxAAL5 *queue, uint8 *payload);

#if 1 /* SID_MASK == 0xFFF */
#define REORDERING_LENGTH 2200 /* allows to store 33 msecs at 20 Mbps */
#else
#define REORDERING_LENGTH 248 /* allows to store 33 msecs at 20 Mbps */
#endif
typedef struct ReorderingQueue ReorderingQueue;
struct ReorderingQueue
{
  AtmCell   cells[REORDERING_LENGTH];
  RxAAL5   *rxaal5;
  uint8     avail[(REORDERING_LENGTH+7)>>3]; /* bitmap indicating whether a
                                              * cell is present at that
                                              location */
  uint32    activeLines;
  uint32    maxSid; /* will be checked on regular base and a force flush will
                     * be called */
  uint32    prevMaxSid;
  uint32    startId; /* sequenceId corresponding to the first cell in the
                      * buffer */
  int       firstCellIndex; /* index of the first cell in the FIFO */
  uint32    sidMask; /* used to perform all modulo operations */

  uint32    cellNumber; /* indicates number of cells present in the reordering
                         * queue */
  uint32    consecutiveOutOfOrder; /* indicates the number of consecutive
                                      cells that arrive with out of order sid */
  uint32    lastSeqId[MAX_BONDED_LINES];

  int outOfOrder;
  int outOfOrderCompleteFlush;
  int outOfRangeSid;
  int lineFlush;
  int missingCell;
  int smallNegative;
};

void ReorderingQueue_init(ReorderingQueue *queue, RxAAL5 *aal5, uint32 sidMask);

/* function will call RxAal5 if necessary */
void ReorderingQueue_pushCell(ReorderingQueue *queue, uint8 *cell, int lineId);
void ReorderingQueue_addNewLine(ReorderingQueue *objp, int lineId);
void ReorderingQueue_initMinReceivedSid(ReorderingQueue *objp, int startId);
uint32 ReorderingQueue_recomputeMinReceivedSid(ReorderingQueue *objp);
void ReorderingQueue_handle(ReorderingQueue *objp,int minCellNumber);


#define AVG_PERIOD_LOG 5
typedef struct TimeDelta TimeDelta;
struct TimeDelta
{
#if 0
  int    lastDeltas[1<<AVG_PERIOD_LOG]; /* keep last received delta */
  int    currentAvg; /* sum of 24LSB of all lastDeltas */
  int    oldestMeas; /* -1 meas never init */
  int    currentDelta;
#else
  int    runningAverage;
  int    currentDelta;
#endif
};

void TimeDelta_init(TimeDelta *obj);
void TimeDelta_newDelta(TimeDelta *obj,int newDelta);
#define TimeDelta_getCurrentDelta(objp) ((objp)->currentDelta)


typedef struct LineInfo LineInfo;
struct LineInfo
{
  MacAddress mac; /* mac address of the slave CPE */
  int        allocated; /* 0 indicates not associated with slave CPE */
  int        dbId;
  int        usBitRate; /* in kbps */
  int        dsBitRate; /* in kbps  == 0 means no showtime */
  int        inUse; /* meas we should send traffic over the line */
  int        usecTx; /* number of micro seconds necessary to Tx 1 ATM cell at
                      * 98% load */
  int        atmReady; /* atm cell can be sent to the CPE */
  uint32     lastTxElapsed; /* elapsed usec between last atmReady and vt
                             * (virtual time) */
  
  TimeDelta  cpeTimeDelta;
  uint32     lastCPErepTimestamp;
  
  /* ASM info  (use a global prepared ASM) */
  int asmReady; /* indicate an asm should be sent on this line  */
  uint32 lastAsmElapsed; /* elapsed time since asmReady has been set */
  int usecTxAsm; /* number of usec to send a asm => reflect asm frequency
                    it is about 100 times the usecTx
                 */

};


/* Device information structures and definitions of the device types */
#define TYPE_AAL5FRAMER       (1 << 0)
#define TYPE_BONDING_SLAVE    (1 << 1)

#define TRAINING_STATE_IDLE                    0
#define TRAINING_STATE_G994_TRAINING           1
#define TRAINING_STATE_G992_STARTED            2
#define TRAINING_STATE_G992_CHANNEL_ANALYSIS   3
#define TRAINING_STATE_G992_MESSAGE_EXCHANGE   4
#define TRAINING_STATE_SHOWTIME                5
#define TRAINING_STATE_UNKNOWN                 6

/* note two bytes reserved are placed bewteen first mac and startof
   CpeStateRep */

typedef struct CpeDeviceInfo CpeDeviceInfo;
struct  CpeDeviceInfo 
{
  uint32 masterTimeStamp;
  uint32 slaveTimeStamp;
  uint32 dsRate;
  uint32 usRate;
  uint16 failures;
  uint16 trainingState;
};

typedef struct CpeDeviceInfoRep CpeDeviceInfoRep;
struct CpeDeviceInfoRep
{
  uint32 deviceType;
  CpeDeviceInfo info;
};

// ATM_VCC_BOND_CFG contains configuration fields for an ATM Virtual Channel
// Connection (VCC).
#define ID_ATM_VCC_BOND_CFG                 2
#define TX_Q_BOND_PARM_SIZE                 3

typedef struct AtmVccBondCfg
{
    UINT32 ulStructureId;
    UINT32 ulAalType;
    UINT32 ulAtmVclAdminStatus;
    UINT32 ulAtmVclOperStatus;
    UINT32 ulAtmVclLastChange;
    UINT32 ulAtmVclReceiveTrafficDescrIndex;
    UINT32 ulAtmVclTransmitTrafficDescrIndex;
    UINT32 ulTransmitQParmsSize;
    ATM_VCC_TRANSMIT_QUEUE_PARMS TransmitQParms[TX_Q_BOND_PARM_SIZE];
    union
    {
        ATM_AAL5_VCC_CFG Aal5Cfg;
        ATM_AAL2_VCC_CFG Aal2Cfg; // [BCM635x Only]
        ATM_AAL0_VCC_CFG Aal0Cfg;
    } u;
} ATM_VCC_BOND_CFG, *PATM_VCC_BOND_CFG;

typedef struct AtmVccEntry AtmVccEntry;
struct AtmVccEntry 
{
  ATM_VCC_ADDR addr;
  ATM_VCC_BOND_CFG cfg;
  ATM_TRAFFIC_DESCR_PARM_ENTRY trafficDescr;
};


typedef struct getDeviceInformationRep getDeviceInformationRep;
struct getDeviceInformationRep
{
  uint32 deviceType;
  AtmVccEntry vcc[8];
  uint32 bonding_bypass;
};

typedef struct imageUpgradeReq imageUpgradeReq;
struct imageUpgradeReq
{
  uint32 ipAddress;
  uint16 port;
  uint16 reserved;
  uint8  fileName[128];
};

typedef struct imageUpgradeRep imageUpgradeRep;
struct imageUpgradeRep
{
  uint32 status;
  uint32 error;
};

#define SLAVE_NO_RETRAIN        0
#define SLAVE_RETRAIN_LOF       1
#define SLAVE_RETRAIN_LOS       2
#define SLAVE_RETRAIN_LOP       3
#define SLAVE_RETRAIN_LOM       4
#define SLAVE_RETRAIN_OTHER     5
#define SLAVE_RETRAIN_UNKNOWN   6

typedef struct slaveRatesInfo slaveRatesInfo;
struct slaveRatesInfo
{
  uint32 slaveUSRate;
  uint32 slaveDSRate;
  uint16 retrain;
  uint16 trainingState;
};

typedef struct slavesState slavesState;
struct slavesState
{
  uint32 aggregateUSRate;
  uint32 aggregateDSRate;
  slaveRatesInfo slavesRates[MAX_BONDED_LINES];
  uint32 slavesUSState;
  uint32 slavesDSState;
};

void LineInfo_init(LineInfo *line);
uint32 LineInfo_changeState(LineInfo *line, CpeDeviceInfo *newState);

uint32 LineInfo_getLocalRxState(LineInfo *objp);
uint32 LineInfo_getLocalTxState(LineInfo *objp,uint32 remoteRxState);

void LineInfo_print(LineInfo *line,int fd);
void LineInfo_resetBonding(LineInfo *line);



/* ASM payload */
typedef struct Asm Asm;
struct Asm
{
  uint8  msgType; /* 00 = 12 bit SID format, 01 = 8 bit SID format */
  uint8  asmId;
  uint8 txLinkId;/* 1 bit insufficient buffer - 4 LSB link id */
  uint8  txLinkNumber; 
  uint16  rxLinkStatus[4];
  uint16  txLinkStatus[4];
  uint16 groupId;
  uint8  rxAsmStatus[4];
  uint8  groupLostCell;
  uint8  reservedA;
  uint32 timestamp;
  uint16 delay;
  uint16 appliedDelay;
  uint16 reservedC[3];
  uint16 msgLen; /* should be 0x2800 */
  uint32 crc;
};


#define LINK_BEING_RESET   1 /* do not klnow link id/bdId. no ASM send on this
                           link. waiting for a bdId==0. Or after a timeout for
                           any BdId */

#define LINK_ID_KNOWN      2 /* we have received a BDID=0. Known waiting for the
                           correct BdId. ASM is send on this link */

#define LINK_ACTIVE        3 /* BdId known, should never get again a BdId = 0 on
                         * this link */

typedef struct LineMapping LineMapping;
struct LineMapping
{
  int localIdToRemoteId;
  /* info to compute the differential delay */
  TimeDelta     differentialDelay;
  uint16        requestedDelay; /* delay to introduce on this line in 100 usecs */
  uint32        lastAsmTimestamp;
  int           receivedCells;
};



#define MAX_VP_VC        8
typedef struct BondingMaster BondingMaster;
struct BondingMaster
{
  uint32     knownMapping; /* indicates which are the known mapping */
  int        remoteIdToLocal[MAX_BONDED_LINES]; /* -1 means it is unknown */
  LineMapping  mapping[MAX_BONDED_LINES];
  LineInfo   lines[MAX_BONDED_LINES];
  uint32     bondingId; /* 0 upon init. Set as soon as known from the master
                         * */
  uint8      asmLastRxId;
  uint8      resetState;
  int        numberOfLines; /* set as soon an ASM is received */
  uint16     remoteTxState; 
  uint16     remoteRxState;
  struct timeval resetTimeout;
  TxAAL5     txAal5;
  RxAAL5     rxAal5[MAX_VP_VC];
  ReorderingQueue reorderingQueue;
  Asm        asmCell; /* contains the ASM information to be sent to remote side */ 
  uint32     vpvc;
  uint32     vt; /* virtual time */

  /* leaky bucket for CPE state ?? */
  int nextLineToPoll; /* perform a polling every 150 msec - line -1 =>
                       * broadcast */
  uint32     timeSinceLastPoll; /* usec */

  uint32 minTransmitTime; /* min time to transmit an ATM cell across the
                             different lines */
  uint32 timeInterval;    /* MIN between minTransmit time and x usec */

  int wrongASMcrc;

  int txCell;
  int rxCell;

  
  /* measure differential delay between link */
  int refLink; /* link that is known as the reference link to measure the
                * end-to-end differential delay */
  int lastPropagationRefDelay; /* last propoagation delay measured on the ref
                                * link */
  int refDiffDelay; /* fixed differential delay on the ref link */
  
};

void BondingMaster_init(BondingMaster *master, uint32 vpvc);
int BondingMaster_newAsm(BondingMaster *master, Asm *asmp,int localLineId);
int  BondingMaster_getLocalId(BondingMaster *master, MacAddress *receivedMac);
void BondingMaster_printState(BondingMaster *master, int level,int reset);



/* next function returns 1 if packet has been handled by the function */
int  master_analysePrivateIncomingFrame(BondingMaster *master, uint8 *framep,int size);

void master_sendPendingPackets(BondingMaster *master,int maxLoops);
void master_checkPollingCPE(BondingMaster *master);
void BondingMaster_global_init(void);

void checkMaster_data(int i);
void EnterBonding(uint32 usec);
void ExitBonding(uint32 usec);
 

#define TASK_NUMBER 11
#define SEND_PENDINGPACKETS 0
#define POLLING_CPE     1
#define ANALYSE_PRIVATE 2
#define AAL5_SENDFRAME  3
#define OTHER           4
#define REORDERING_TOOLATE 5
#define REORDERING_TOOFAR  6
#define REORDERING_CPY     7
#define MINSID_HANDLING    8
#define REORDERING         9
#define ATM_CELL          10

void LocalNewPoll(void);
void LocalStartTask(int taskId);

#if 1
#define  NewPoll()     0
#define  StartTask(a)  0
#else
#define  NewPoll       LocalNewPoll
#define  StartTask     LocalStartTask
#endif



extern BondingMaster master;
extern int g_bondingOn;
extern int g_bondingTriggerPacket;


#define BYTESWAP16(a) ((uint16)((((uint16)(a)<<8) | ((uint16)(a)>>8)) & 0xffff))
#define BYTESWAP32(a) \
((uint32)((a)<<24) | (((uint32)(a)<<8)&0xff0000) | (((uint32)(a)>>8)&0xff00) | ((uint32)(a)>>24))


#ifdef BIG_ENDIAN_LAYOUT
#define TO_FROM_BE_32(a)  (a)
#define TO_FROM_LE_32(a)  (BYTESWAP32(a))
#define TO_FROM_BE_16(a)  (a)
#define TO_FROM_LE_16(a)  (BYTESWAP16(a))
#else
#define TO_FROM_LE_32(a)  (a)
#define TO_FROM_BE_32(a)  (BYTESWAP32(a))
#define TO_FROM_LE_16(a)  (a)
#define TO_FROM_BE_16(a)  (BYTESWAP16(a))
#endif

#endif
